@@ -24,45 +24,38 @@
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationInterchange <- function(x, y){
-	N=length(x) #==length(y)
-	x<-y[order(x)]
-	result <- .C("permutationDistanceInterchange", as.integer(x), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	(N-result$ret) / (N-1) 
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	#x<-y[order(x)]
+	result <- .Call("permutationDistanceInterchange", as.integer(x),as.integer(y), PACKAGE="CEGO")
+	(N-result) / (N-1) 
 }
 
 ###################################################################################
 #' Longest Common Subsequence Distance for Permutations
 #' 
-#' Distance of permutations. Based on the longest sequence (breaks allowed) of permutation elements 
-#' in increasing order that both permutations have in common.
-#' This distance is also called Ulam's metric and is the same
-#' as the distance measure based on the insert edit-operation (\code{\link{distancePermutationInsert}}). Note, that
-#' the implementations differ. Insert distance scales better for longer permutations, whereas LCSeq is faster for shorter
-#' permutations.
+#' DEPRECATED, see \code{\link{distancePermutationInsert}}. 
 #'
 #' @param x first permutation (integer vector)
 #' @param y second permutation (integer vector)
 #'
 #' @return numeric distance value \deqn{d(x,y)}, scaled to values between 0 and 1 (based on the maximum possible distance between two permutations)
 #'
-#' @references Hirschberg, Daniel S. "A linear space algorithm for computing maximal common subsequences." Communications of the ACM 18.6 (1975): 341-343.
-#'
-#' @examples
-#' x <- 1:5
-#' y <- c(1,2,5,3,4)
-#' distancePermutationLCSeq(x,y)
-#' p <- replicate(10,sample(1:5),simplify=FALSE)
-#' distanceMatrix(p,distancePermutationLCSeq)
-#'
 #' @export
-#' @useDynLib CEGO
+#' @keywords internal
 ###################################################################################
-distancePermutationLCSeq <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceLongestCommonSubsequence", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	(N-result$ret)/(N-1) # N-1 is for PERMUTATIONS only, because two permutations have always at least a common string/sequence of length one. 
+distancePermutationLCSeq<- function(x, y){
+	.Deprecated("distancePermutationInsert")
+	#N <- length(x)
+	#if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	#result <- .Call("permutationDistanceLongestCommonSubsequence", as.integer(x),as.integer(y), PACKAGE="CEGO")
+	#(N-result)/(N-1) # N-1 is for PERMUTATIONS only, because two permutations have always at least a common string/sequence of length one. 
                   # this will be different for strings where two strings may have no single character in common. permutations always contain all 
                   # of N characters.
+	### 
+	## insert is identical but faster
+	###
+	distancePermutationInsert(x,y)									
 }
 
 ###################################################################################
@@ -88,9 +81,10 @@ distancePermutationLCSeq <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationLCStr <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceLongestCommonSubstring", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	(N-result$ret)/(N-1)
+	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	result <- .Call("permutationDistanceLongestCommonSubstring",  as.integer(x),as.integer(y), PACKAGE="CEGO") 
+	(N-result)/(N-1)
 }
 
 ###################################################################################
@@ -116,9 +110,9 @@ distancePermutationLCStr <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationLevenshtein <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceLevenshtein", as.integer(x), as.integer(y), as.integer(N+1), ret=integer(1), PACKAGE="CEGO")
-	result$ret/N
+	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	.Call("permutationDistanceLevenshtein",  as.integer(x),as.integer(y), PACKAGE="CEGO") / N
 }
 
 ###################################################################################
@@ -145,9 +139,10 @@ distancePermutationLevenshtein <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationSwap <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceSwap", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	2*result$ret / (N^2 - N)
+	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	result <- .Call("permutationDistanceSwap", as.integer(x),as.integer(y),  PACKAGE="CEGO")
+	2*result / (N^2 - N)
 }
 
 ###################################################################################
@@ -174,9 +169,10 @@ distancePermutationSwap <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationR <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceR", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	result$ret / (N - 1)
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	result <- .Call("permutationDistanceR", as.integer(x), as.integer(y), PACKAGE="CEGO")
+	result / (N - 1)
 }
 
 ###################################################################################
@@ -203,9 +199,10 @@ distancePermutationR <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationAdjacency <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistanceAdjacency", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	(N-result$ret-1) / (N - 1)
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	result <- .Call("permutationDistanceAdjacency", as.integer(x), as.integer(y), PACKAGE="CEGO")
+	(N-result-1) / (N - 1)
 }
 
 ###################################################################################
@@ -232,9 +229,9 @@ distancePermutationAdjacency <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationPosition <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistancePosition", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	dis <- result$ret
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	dis <- .Call("permutationDistancePosition", as.integer(x), as.integer(y), PACKAGE="CEGO")
 	if(N%%2) #scale to [0;1] in case of odd N
 		dis <- dis / ((N^2-1)/ 2)
 	else #scale to [0;1] in case of even N
@@ -266,9 +263,10 @@ distancePermutationPosition <- function(x, y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationPosition2 <- function(x, y){
-	N=length(x) #==length(y)
-	result <- .C("permutationDistancePosition2", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	result$ret / ((N^3-N)/3) 
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	result <- .Call("permutationDistancePosition2", as.integer(x), as.integer(y), PACKAGE="CEGO")
+	result / ((N^3-N)/3) 
 }
 
 ###################################################################################
@@ -292,11 +290,15 @@ distancePermutationPosition2 <- function(x, y){
 #' @export
 ###################################################################################
 distancePermutationHamming <- function(x, y){
-	#N=length(x) #==length(y)
-	#result <- .C("permutationDistanceHamming", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO") #this works,but is actually slower than a pure R implementation, as below. at least for permutation length 30
-	#result$ret / N
-	sum(x != y)/length(x)
+	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	sum(x != y)/N
 }
+#distancePermutationHamming <- function(x, y){
+#	N <- length(x)
+#	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+#	.Call("permutationDistanceHamming", as.integer(x), as.integer(y),PACKAGE="CEGO") / N #this works,but is actually slower than a pure R implementation
+#}
 
 ###################################################################################
 #' Euclidean Distance for Permutations
@@ -321,9 +323,10 @@ distancePermutationHamming <- function(x, y){
 #' @export
 ###################################################################################
 distancePermutationEuclidean <- function(x, y){
-	N=length(x) #==length(y)
-	#result <- .C("permutationDistanceEuclidean", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO") #this works,but is actually slower than a pure R implementation, as below. at least for permutation length 30
-	#mdis <- sqrt(result$ret)
+	N<-length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	#result <- .Call("permutationDistanceEuclidean", as.integer(x), as.integer(y), PACKAGE="CEGO") #this works,but is actually slower than a pure R implementation, as below. at least for permutation length 30
+	#mdis <- sqrt(result)
 	mdis <- sqrt(sum((x-y)^2))
 	if(N%%2){ #scale to [0;1] in case of odd N
 		n=(N-1)/2
@@ -360,6 +363,7 @@ distancePermutationEuclidean <- function(x, y){
 ###################################################################################
 distancePermutationManhattan <- function(x, y){
 	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
 	mdis<-sum(abs(x-y))
 	if(N%%2) #scale to [0;1] in case of odd N
 		dis <- mdis / ((N^2-1)/ 2)
@@ -391,6 +395,7 @@ distancePermutationManhattan <- function(x, y){
 ###################################################################################
 distancePermutationChebyshev <- function(x, y){
 	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
 	max(abs(x-y)) / (N-1)
 }
 
@@ -421,8 +426,8 @@ distancePermutationChebyshev <- function(x, y){
 ###################################################################################
 distancePermutationLee <- function(x,y){
 	N <- length(x)
-	result <- .C("permutationDistanceLee", as.integer(x), as.integer(y), as.integer(N), ret=integer(1), PACKAGE="CEGO") #this works,but is actually slower than a pure R implementation, as below. at least for permutation length 30
-	mdis <- result$ret
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	mdis <- .Call("permutationDistanceLee", as.integer(x), as.integer(y), PACKAGE="CEGO") #this works,but is actually slower than a pure R implementation, as below. at least for permutation length 30
 	#mdis <- sum(pmin(abs(x-y),N-abs(x-y)))
 	if(N%%2) #scale to [0;1] in case of odd N
 		dis <- mdis / ((N^2-1)/ 2)
@@ -431,17 +436,14 @@ distancePermutationLee <- function(x,y){
 	dis
 }
 
-
 ###################################################################################
 #' Insert Distance for Permutations
 #' 
-#' The Insert Distance is an edit distance. It counts the minimum number of insert operations
-#' required to transform one permutation into another. An insert operation shifts one element to a new position.
-#' All other elements move accordingly to make place for the element. E.g., the following shows a single insert move that
-#' sorts the corresponding permutation: 1 4 2 3 5 -> 1 2 3 4 5. This distance is also called Ulam's metric and is the same
-#' as the distance measure based on the longest common subsequence (\code{\link{distancePermutationLCSeq}}). Note, that
-#' the implementations differ. Insert distance scales better for longer permutations, whereas LCSeq works better for shorter
-#' permutations.
+#' The Insert Distance is an edit distance. It counts the minimum number of delete/insert operations
+#' required to transform one permutation into another. A delete/insert operation shifts one element to a new position.
+#' All other elements move accordingly to make place for the element. E.g., the following shows a single delete/insert move that
+#' sorts the corresponding permutation: 1 4 2 3 5 -> 1 2 3 4 5. This distance is also called Ulam's metric and can as well
+#' be interpreted to be based on the longest common subsequence of two permutations. 
 #'
 #' @param x first permutation (integer vector)
 #' @param y second permutation (integer vector)
@@ -462,35 +464,11 @@ distancePermutationLee <- function(x,y){
 #' @useDynLib CEGO
 ###################################################################################
 distancePermutationInsert <- function(x, y){
-	N=length(x) #==length(y)
-	x<-order(y)[x] #Important note: this line means "p1 * p2^{-1}". Schiavinotto say it should be  "p2^{-1} * p1" . That seems to be a typo/error. (of course it does not matter which permutation is is inverted. but the inverted permutation should ALWAYS be at the right.)
-	result <- .C("longestIncreasingSubsequence", as.integer(x), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	(N-result$ret)/(N-1)
-}
-
-###################################################################################
-#' Longest Increasing Subsequence
-#'
-#' Calculate the longest increasing subsequence (LIS) in a vector (e.g., a permutation).
-#'
-#' @param x integer vector
-#'
-#' @references Wikipedia contributors, "Longest increasing subsequence", Wikipedia, The Free Encyclopedia, 12 November 2014, 19:38 UTC, <http://en.wikipedia.org/w/index.php?title=Longest_increasing_subsequence&oldid=633565014> [accessed 13 November 2014] 
-#'
-#' @examples
-#' longestIncreasingSubsequence(1:5) 
-#' longestIncreasingSubsequence(5:1)
-#' longestIncreasingSubsequence(c(1,2,4,3,5))
-#' longestIncreasingSubsequence(c(1,2,4,3,5,1,2,4,3,2,9)) 
-#'
-#' @export
-#' @useDynLib CEGO
-###################################################################################
-longestIncreasingSubsequence <- function(x){
-	N=length(x) #==length(y)
-	
-	result <- .C("longestIncreasingSubsequence", as.integer(x), as.integer(N), ret=integer(1), PACKAGE="CEGO")
-	result$ret
+	N<-length(x) 
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
+	#x<-order(y)[x] #Important note: this line means "p1 * p2^{-1}". Schiavinotto say it should be  "p2^{-1} * p1" . That seems to be a typo/error. (of course it does not matter which permutation is is inverted. but the inverted permutation should be at the right.). Is now calculated in c code
+	result <- .Call("permutationDistanceInsert", as.integer(x), as.integer(y), PACKAGE="CEGO")
+	(N-result)/(N-1)
 }
 
 ###################################################################################
@@ -518,6 +496,7 @@ longestIncreasingSubsequence <- function(x){
 ###################################################################################
 distancePermutationCos <- function(x,y){
 	N=length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
 	as.numeric(1-((x%*%y)-(N*(N+1)*(N+2)/6))/((N^3-N)/6)) # |a| = |b| because permutation -> |a| * |a| = sum_{i=1}^n i^2 = (N*(N+1)*(2*N+1)/6)
 }#the minus factor is tetrahedral pyramid
 
@@ -551,6 +530,7 @@ distancePermutationCos <- function(x,y){
 ###################################################################################
 distancePermutationLex <- function(x,y){
 	N <- length(x)
+	if(N!=length(y)|!is.numeric(x)|!is.numeric(y)) stop("Incorrect input to distance function, only permutations of same length are allowed.")
 	abs(lexicographicPermutationOrderNumber(x) - lexicographicPermutationOrderNumber(y)) / (factorial(N)-1)
 }
 
@@ -583,6 +563,7 @@ distancePermutationLex <- function(x,y){
 ###################################################################################
 lexicographicPermutationOrderNumber <- function(x){
 	N <- length(x)
-	result <- .C("lexPermOrder", as.integer(x), as.integer(N), ret=integer(N), PACKAGE="CEGO")
-	sum(result$ret*factorial((N-1):0))+1
-}
\ No newline at end of file
+	if(!is.numeric(x)) stop("Incorrect input to lexicographicPermutationOrderNumber, should be permutation.")
+	result <- .Call("lexPermOrder", as.integer(x), PACKAGE="CEGO")
+	sum(result*factorial((N-1):0))+1
+}
